#!/usr/bin/env python

'''Event Manager query tool'''
__revision__ = '$Revision: 1.13 $'

from getopt import getopt, GetoptError
from sys import argv
from elementtree.ElementTree import XML, tostring, Element, SubElement

from sss.ssslib import comm_lib, LookupError as SSSLookupError, ProtocolError

if __name__ == '__main__':
    try:
        (opts, args) = getopt(argv[1:], 'esdc:m:n:a:')
    except GetoptError, m:
        print m
        print "Usage: eminfo.py [-e] [-s] [-d] [-c component] [-m msg] [-a data] [-n notify]"
        raise SystemExit, 1

    spec = {}
    spec['component'] = ''
    spec['msg'] = ''
    spec['data'] = ''
    spec['notify'] = ''

    for opt in opts:
        if opt[0] == '-c':
            spec['component'] = opt[1]
        elif opt[0] == '-m':
            spec['msg'] = opt[1]
        elif opt[0] == '-n':
            spec['notify'] = opt[1]
        elif opt[0] == '-a':
            spec['data'] = opt[1]

    if '-s' in argv:
        msg = Element("get-subscription")
        sub = SubElement(msg, "subscription")
        for c in ['component','msg','data','notify']:
            tmp = SubElement(sub, c)
            if spec[c] == '':
                tmp.attrib['match'] = 'false'
        for i in ['matches','delivered','failures','last']:
            tmp = SubElement(sub, i)
            tmp.attrib['match'] = 'false'
        emsg = tostring(msg)
    elif '-e' in argv:
        emsg = "<get-event><event><component>%s</component><msg>%s</msg><data>%s</data><time match='false'/></event></get-event>" % (spec['component'], spec['msg'], spec['data'])
    else:
        print "Usage: eminfo.py [-e] [-s] [-d] [-c component]"
        print "one of -e or -s are required"
        raise SystemExit, 1

    debug = len([x for x in argv if x=='-d'])
    c = comm_lib(debug=debug)
    try:
        h = c.ClientInit("event-manager")
    except SSSLookupError:
        print "Event Manager not registered"
        raise SystemExit, 1
    except ProtocolError:
        print "Protocol error"
        raise SystemExit, 1
    c.SendMessage(h, emsg)
    resp = c.RecvMessage(h)
    c.ClientClose(h)
    element = XML(resp)
    if element.tag == 'Error':
        print "Server failure"
    else:
        for item in element:
            print tostring(item)
