#!/usr/bin/env python
from cPickle import dumps, loads
from string import split,join
from os import fork, close
from select import select
from signal import signal, SIGINT
from syslog import syslog, LOG_ERR
from elementtree.ElementTree import Element, SubElement, tostring
from time import strftime, localtime
from getopt import getopt, GetoptError
from sys import argv,exit

from sss.server import Server, BufferedPipe
from sss.ssslib import comm_lib
from sss.lrs import LRDataSet as DataSet, LRData as Data

class sss_build(Server):
    __component__='build-system'
    __implementation__='chiba-bs'   
    __statefields__ = ['nodesw','tagdata','userdata']
    __dispatch__= {    'get-node-software':"nodesw.Get",
                       'add-node-software':"XAddNS",
                       'set-node-software':"XSetNS",
                       'del-node-software':"nodesw.Del",
                       'get-tag':"tagdata.Get",
                       'set-tag':"XSetTag",
                       'add-tag':"tagdata.Add",
                       'del-tag':"tagdata.Del",
                       'get-users':"userdata.Get",
                       'set-users':"XSetUser",
                       'add-users':"userdata.Add",
                       'del-users':"userdata.Del",
                       'build-node':"Build"}
    __validate__ = 1
                       
    def __setup__(self):
        self.nodesw = DataSet("node-software-set", "node-software", Data, None, False)
        self.tagdata = DataSet("tags", "tag", Data, None, False)
        self.userdata = DataSet('users', 'user', Data, None, False)

    def XAddNS(self,xml,(peer,port)):
        # Set defaults if missing
        for x in xml:
            for at in ['mayor','kernel','kernel_opts','image','action','class']:
                if x.find(at) == None:
                    if at=='action':
                        SubElement(x, at)
                        x.find(at).text = 'inventory'
                    else:
                        SubElement(x, at)
                        x.find(at).text = ''
        return self.nodesw.Add(xml, (peer,port), None)

    def XSetNS(self,xml,(peer,port)):
        return self.SetGeneral(self.nodesw, xml, (peer,port))

    def XSetTag(self,xml,(peer,port)):
        return self.SetGeneral(self.tagdata, xml, (peer,port))

    def XSetUser(self,xml,(peer,port)):
        return self.SetGeneral(self.userdata, xml, (peer,port))

    def SetGeneral(self, data, xml, (peer,port)):
        self.changed_nodes = []
        result = data.Get(xml,(peer,port), self.SetNS)
        # Send notification of state change
        self.evt.WriteMessage(dumps(self.changed_nodes))
        return result

    def SetNS(self, item, attrib):
        del attrib['node-software']
        for x in attrib.keys():
            if item.element.find(x) != None and item.element.find(x).text != attrib[x]:
                item.element.find(x).text = attrib[x]
                if not (item.element.find('node').text in self.changed_nodes):
                    self.changed_nodes.append(item.element.find('node').text)
        
    def Build(self,xml, (peer,port)):
        pass

    def ChildSigHand(self,signum,frame):
        exit(0)
    
    def __setupChildren__(self):
        self.evt = BufferedPipe()
        self.pipes.append(self.evt)
        pid=fork()
        if pid==0:
            signal(SIGINT, self.ChildSigHand)
            self.readsockets=[self.evt.r]
            self.socketdispatch={self.evt.r:self.ProcessChanges}
            self.ServeForever()
        else:
            close(self.evt.r)
            return [pid]

    def ProcessChanges(self,pipe,inl=None):
        while 1:
            try:
                (infd,out,x) = select([self.evt.r],[],[],5)
            # Catch and disregard the traceback on child process exit
            except:
                pass
            if infd != []:
                self.evt.ReadMessages()

            while self.evt.buffer:
                changed_nodes=loads(self.evt.ReadMessage())
        
                # Send state changed message
                doc=Element('add-event')
                for node in changed_nodes:
                    se = SubElement(doc, 'event')
                    SubElement(se, "component")
                    se.find("component").text = self.__component__
                    SubElement(se, "msg")
                    se.find("msg").text = "state-changed"
                    SubElement(se, "date")
                    se.find("data").text = node
                    SubElement(se, "time")
                    se.find("time").text = strftime("%c",localtime())
                if len(doc) > 0:
                    try:
                        c=comm_lib()
                        h=c.ClientInit('event-manager')
                        c.SendMessage(h,tostring(doc))
                        c.RecvMessage(h)
                        c.ClientClose(h)
                    except:
                        syslog(LOG_ERR, "Failed to send state-changed event")
        
                    del doc
        return 0

if __name__ == '__main__':
    try:
        (opts,args)=getopt(argv[1:],'d',['daemon='])
    except GetoptError,msg:
        print "%s\nUsage:\nbs.py [-d] [--daemon <pidfile>]"%(msg)
        exit(1)
    daemon=[x[1] for x in opts if x[0] == '--daemon']
    debug=len([x for x in opts if x[0] == '-d'])
    if daemon:
        from sss.daemonize import daemonize
        daemonize(daemon[0])
    server=sss_build(debug=debug)
    server.ServeForever()
